package com.lordcodes.turtle.sample

import com.lordcodes.turtle.Arguments
import com.lordcodes.turtle.Command
import com.lordcodes.turtle.Executable
import com.lordcodes.turtle.GitCommands
import com.lordcodes.turtle.shellRun
import java.io.File
import java.net.URL

fun main() {
    shellRun {
        val gitLocation = files.which("git") ?: error("git must be installed")
        println("git: $gitLocation")

        println(git.gitAuthorName())

        Command(Executable("mkdir"), Arguments("Test")).executeOrElse { it.message ?: "Failed" }

        command("mkdir", listOf("Test2"))
        command("rm", listOf("-rf", "BLAH"))
        command("rm", listOf("-rf", "Test", "Test2"))

        println(Git.add(listOf()).executeOrElse { it.message ?: "Error" })

        "Finished"
    }
}

fun GitCommands.gitAuthorName() = gitCommand(listOf("--no-pager", "show", "-s", "--format=%an"))

object Git {
    val executable = Executable("git", URL("https://git-scm.com/docs"))

    enum class Action {
        Init, Clone, Add;

        fun command(): Command = Command(executable, Arguments(name.lowercase()))
    }

    fun clone(url: URL, destination: File? = null): Command =
        Action.Clone.command() + Arguments(url, destination)

    fun init(): Command = Action.Init.command()

    fun add(files: List<File>): Command = Action.Add.command() + Arguments(files)
}
